// EnbanFukusyaDlg.cpp : Cve[V t@C
//

#include "stdafx.h"
#include "EnbanFukusya.h"
#include "EnbanFukusyaDlg.h"
#include "ThemeMessageDlg.h"
#include "SettingDlg.h"
#include "ThemeSelectDlg.h"
#include <GraphicUtil.h>

#include "aspi.h"
#include <scan.h>
#include <ui.h>
#include <cmd.h>
#include <copydisc.h>
#include <cmdlog.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// AvP[Ṽo[WŎgĂ CAboutDlg _CAO

class CAboutDlg : public CThemeDialog
{
public:
	CAboutDlg();

// _CAO f[^
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	CThemeLabel	m_StcCopyright;
	CThemeLabel	m_StcVersion;
	CThemeButton	m_BtnOK;
	//}}AFX_DATA

	// ClassWizard ͉z֐̃I[o[Ch𐶐܂
	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV ̃T|[g
	//}}AFX_VIRTUAL

// Cve[V
protected:
	//{{AFX_MSG(CAboutDlg)
	virtual BOOL OnInitDialog();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CThemeDialog(CAboutDlg::IDD, NULL)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	DDX_Control(pDX, IDC_STC_COPYRIGHT, m_StcCopyright);
	DDX_Control(pDX, IDC_STC_VERSION, m_StcVersion);
	DDX_Control(pDX, IDOK, m_BtnOK);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BOOL CAboutDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	// TODO: ̈ʒuɏ̕⑫ǉĂ
	m_BtnOK.SetTheme(GetTheme());
	m_StcCopyright.SetTheme(GetTheme());
	m_StcVersion.SetTheme(GetTheme());
	return TRUE;  // Rg[ɃtH[JXݒ肵ȂƂA߂l TRUE ƂȂ܂
	              // O: OCX vpeB y[W̖߂l FALSE ƂȂ܂
}


/////////////////////////////////////////////////////////////////////////////
// CEnbanFukusyaDlg _CAO

CEnbanFukusyaDlg::CEnbanFukusyaDlg(CWnd* pParent /*=NULL*/)
	: CThemeDialog(CEnbanFukusyaDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CEnbanFukusyaDlg)
	//}}AFX_DATA_INIT
	// : LoadIcon  Win32  DestroyIcon ̃TuV[PXv܂B
	//m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
	m_hIcon = AfxGetApp()->LoadIcon(IDI_APPICON);
	m_pReaderDrives = NULL;
	m_pWriterDrives = NULL;
	m_bRunning = FALSE;
	m_nWindowSizeStep = -1;
	m_bScanDone = FALSE;
	m_nAllWritableDrives = -1;
	m_nReadDriveISO = -1;
	m_nWriteDriveISO = -1;
	m_nReadDriveNet = -1;
	m_nWriteDriveNet = -1;
}

void CEnbanFukusyaDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CEnbanFukusyaDlg)
	DDX_Control(pDX, IDC_STC_INFO, m_StcInfo);
	DDX_Control(pDX, IDC_STC_WRITER, m_StcWriter);
	DDX_Control(pDX, IDC_STC_READER, m_StcReader);
	DDX_Control(pDX, IDC_STC_PROGRESS2, m_StcProgress2);
	DDX_Control(pDX, IDC_STC_PROGRESS1, m_StcProgress1);
	DDX_Control(pDX, IDC_COMBO2, m_CmbWriter);
	DDX_Control(pDX, IDC_COMBO1, m_CmbReader);
	DDX_Control(pDX, IDC_PROGRESS2, m_Progress2);
	DDX_Control(pDX, IDC_PROGRESS1, m_Progress1);
	DDX_Control(pDX, IDCANCEL, m_BtnExit);
	DDX_Control(pDX, IDOK, m_BtnOK);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CEnbanFukusyaDlg, CDialog)
	//{{AFX_MSG_MAP(CEnbanFukusyaDlg)
	ON_WM_SYSCOMMAND()
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	//}}AFX_MSG_MAP
	ON_MESSAGE(WM_KICKIDLE, OnKickIdle)
	ON_MESSAGE(WM_USER_UICTL, OnUICtl)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CEnbanFukusyaDlg bZ[W nh

BOOL CEnbanFukusyaDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// "o[W..." j[ڂVXe j[֒ǉ܂B

	// IDM_ABOUTBOX ̓R}h j[͈̔͂łȂ΂Ȃ܂B
	ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
	ASSERT(IDM_ABOUTBOX < 0xF000);

	CMenu* pSysMenu = GetSystemMenu(FALSE);
	if (pSysMenu != NULL){
		CString strAboutMenu, strThemeSelect;
		strAboutMenu.LoadString(IDS_ABOUTBOX);
		strThemeSelect.LoadString(IDS_THEMESELECT);
		if (!strAboutMenu.IsEmpty()){
			pSysMenu->AppendMenu(MF_SEPARATOR);
			pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
			if(!strThemeSelect.IsEmpty()){
				pSysMenu->AppendMenu(MF_STRING, IDM_THEMESELECT, strThemeSelect);
			}
		}
	}

	// ̃_CAOp̃ACRݒ肵܂Bt[[N̓AvP[ṼC
	// EBhE_CAOłȂ͎Iɐݒ肵܂B
	SetIcon(m_hIcon, TRUE);			// 傫ACRݒ
	SetIcon(m_hIcon, FALSE);		// ACRݒ
	
	// TODO: ʂȏs͂̏ꏊɒǉĂB
	LoadIniFile();
	SetWindowText("~Օʉ");
	OpenAspi();
	LoadTheme();
	ResizeWindow(0);

	return TRUE;  // TRUE ԂƃRg[ɐݒ肵tH[JX͎܂B
}

void CEnbanFukusyaDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
	switch(nID & 0xFFF0){
	case IDM_ABOUTBOX:
		{
			CAboutDlg dlgAbout;
			CThemeSet *pThemeClass = ((CEnbanFukusyaApp *)AfxGetApp())->GetThemeClass();
			dlgAbout.SetTheme(pThemeClass);
			dlgAbout.DoModal();
		}
		break;
	case IDM_THEMESELECT:
		{
			CThemeSelectDlg dlgThemeSel;
			CThemeSet *pThemeClass = ((CEnbanFukusyaApp *)AfxGetApp())->GetThemeClass();
			dlgThemeSel.SetTheme(pThemeClass);
			dlgThemeSel.SetThemeNumber(m_nThemeNumber);
			if(dlgThemeSel.DoModal()==IDOK){
				m_nThemeNumber = dlgThemeSel.GetThemeNumber();
				LoadTheme();
				RedrawWindow(NULL, NULL, RDW_INVALIDATE|RDW_FRAME);
				Invalidate();
			}
		}
		break;
	default:
		CDialog::OnSysCommand(nID, lParam);
	}
}

// _CAO{bNXɍŏ{^ǉȂ΁AACR`悷
// R[hȉɋLqKv܂BMFC AvP[V document/view
// fgĂ̂ŁȀ̓t[[Nɂ莩Iɏ܂B

void CEnbanFukusyaDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // `p̃foCX ReLXg

		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

		// NCAg̋`̈̒
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// ACR`悵܂B
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
		DrawLogo();
	}
}

// VXéA[U[ŏEBhEhbOĂԁA
// J[\\邽߂ɂĂяo܂B
HCURSOR CEnbanFukusyaDlg::OnQueryDragIcon()
{
	return (HCURSOR) m_hIcon;
}

void CEnbanFukusyaDlg::LoadTheme()
{
	LOADABLE_THEME_BITMAP ltbButtonNormal		= { IDB_BUTTON_NORMAL, 0, 4, 4, TBF_STRETCH };
	LOADABLE_THEME_BITMAP ltbButtonFocus		= { IDB_BUTTON_FOCUS, 0, 4, 4, TBF_STRETCH };
	LOADABLE_THEME_BITMAP ltbButtonSelect		= { IDB_BUTTON_SELECT, 0, 4, 4, TBF_STRETCH };
	LOADABLE_THEME_BITMAP ltbButtonDisable		= { IDB_BUTTON_DISABLE, 0, 4, 4, TBF_STRETCH };
	LOADABLE_THEME_BITMAP ltbDropButton			= { IDB_DROPDOWN, IDB_DROPDOWN_MASK, 4, 4, TBF_STRETCH };
	LOADABLE_THEME_BITMAP ltbComboBoxBG			= { IDB_COMBOBOX, 0, 2, 2, TBF_TILE };
	LOADABLE_THEME_BITMAP ltbChecked			= { IDB_CHECKED, 0, 4, 4, TBF_STRETCH };
	LOADABLE_THEME_BITMAP ltbUnchecked			= { IDB_UNCHECKED, 0, 4, 4, TBF_STRETCH };
	LOADABLE_THEME_BITMAP ltbProgressBG			= { IDB_PROGRESSBG, 0, 16, 4, TBF_VSTRETCH };
	LOADABLE_THEME_BITMAP ltbProgressFG			= { IDB_PROGRESSFG, 0, 12, 4, TBF_VSTRETCH };
	LOADABLE_THEME_BITMAP ltbDialogBG			= { IDB_DIALOGBG, 0, 16, 0, TBF_TILE };
	LOADABLE_THEME_BITMAP ltbCloseButton		= { IDB_CLOSEBUTTON, 0, 4, 4, TBF_STRETCH };
	LOADABLE_THEME_BITMAP ltbTitleActive		= { IDB_TITLE_ACT, 0, 16, 0, TBF_VSTRETCH };
	LOADABLE_THEME_BITMAP ltbTitleInactive		= { IDB_TITLE_INACT, 0, 16, 0, TBF_VSTRETCH };
	LOADABLE_THEME theme_normal[] = {
		{ TCID_DEFAULT, TTID_FGCOLOR, TSID_NORMAL, THEME_DATA_COLOR(RGB(255,255,200)) },
		{ TCID_DEFAULT, TTID_FGCOLOR, TSID_DISABLED, THEME_DATA_COLOR(RGB(192,192,192)) },
		{ TCID_BUTTON, TTID_BITMAP, TSID_NORMAL, THEME_DATA_BITMAP(&ltbButtonNormal) },
		{ TCID_BUTTON, TTID_BITMAP, TSID_FOCUSED, THEME_DATA_BITMAP(&ltbButtonFocus) },
		{ TCID_BUTTON, TTID_BITMAP, TSID_SELECTED, THEME_DATA_BITMAP(&ltbButtonSelect) },
		{ TCID_BUTTON, TTID_BITMAP, TSID_DISABLED, THEME_DATA_BITMAP(&ltbButtonDisable) },
		{ TCID_CHECKBOX, TTID_CHECKED, TSID_NORMAL, THEME_DATA_BITMAP(&ltbChecked) },
		{ TCID_CHECKBOX, TTID_UNCHECKED, TSID_NORMAL, THEME_DATA_BITMAP(&ltbUnchecked) },
		{ TCID_COMBOBOX, TTID_BITMAP, TSID_NORMAL, THEME_DATA_BITMAP(&ltbComboBoxBG) },
		{ TCID_COMBOBOX, TTID_DROPBUTTON, TSID_NORMAL, THEME_DATA_BITMAP(&ltbDropButton) },
		{ TCID_PROGRESSBAR, TTID_BITMAP, TSID_NORMAL, THEME_DATA_BITMAP(&ltbProgressBG) },
		{ TCID_PROGRESSBAR, TTID_FGBITMAP, TSID_NORMAL, THEME_DATA_BITMAP(&ltbProgressFG) },
		{ TCID_DIALOG, TTID_BGBITMAP, TSID_NORMAL, THEME_DATA_BITMAP(&ltbDialogBG) },
		{ TCID_DIALOG, TTID_CLOSEBUTTON, TSID_NORMAL, THEME_DATA_BITMAP(&ltbCloseButton) },
		{ TCID_DIALOG, TTID_TITLEBITMAP, TSID_NORMAL, THEME_DATA_BITMAP(&ltbTitleActive) },
		{ TCID_DIALOG, TTID_TITLEBITMAP, TSID_DISABLED, THEME_DATA_BITMAP(&ltbTitleInactive) },
		{ TCID_END, 0, 0, NULL }
	};
	/* Aqua */
	LOADABLE_THEME_BITMAP ltbAquaButton      = { IDB_AQUA_BUTTON, IDB_AQUA_BUTTONMASK, 10, 2, TBF_STRETCH };
	LOADABLE_THEME_BITMAP ltbAquaButtonFocus = { IDB_AQUA_BUTTONFOCUS, IDB_AQUA_BUTTONMASK, 10, 2, TBF_STRETCH };
	LOADABLE_THEME_BITMAP ltbAquaCheck       = { IDB_AQUA_CHECKED, IDB_AQUA_CHECKEDMASK, 0, 0, TBF_TILE };
	LOADABLE_THEME_BITMAP ltbAquaUncheck     = { IDB_AQUA_UNCHECKED, IDB_AQUA_UNCHECKEDMASK, 0, 0, TBF_TILE };
	LOADABLE_THEME_BITMAP ltbAquaProgressBG  = { IDB_AQUA_PROGRESSBG, 0, 0, 0, TBF_VSTRETCH };
	LOADABLE_THEME_BITMAP ltbAquaProgressFG  = { IDB_AQUA_PROGRESSFG, 0, 0, 0, TBF_VSTRETCH };
	LOADABLE_THEME_BITMAP ltbAquaDropButton  = { IDB_AQUA_DROPBUTTON, IDB_AQUA_DROPBUTTONMASK, 0, 0, TBF_STRETCH };
	LOADABLE_THEME_BITMAP ltbAquaComboBoxBG  = { IDB_AQUA_COMBOBOXBG, 0, 6, 2, TBF_VSTRETCH };
	LOADABLE_THEME_BITMAP ltbAquaTitleActive = { IDB_AQUA_TITLEACT, 0, 0, 0, TBF_VSTRETCH };
	LOADABLE_THEME_BITMAP ltbAquaTitleInactive  = { IDB_AQUA_TITLEINACT, 0, 0, 0, TBF_VSTRETCH };
	LOADABLE_THEME_BITMAP ltbAquaDialog      = { IDB_AQUA_DIALOGBG, 0, 0, 0, TBF_TILE };
	LOADABLE_THEME_BITMAP ltbAquaCloseButton = { IDB_AQUA_CLOSEBUTTON, IDB_AQUA_CLOSEBUTTONMASK, 0, 0, TBF_STRETCH };
	LOADABLE_THEME_BITMAP ltbAquaMaxButton = { IDB_AQUA_MAXBUTTON, IDB_AQUA_CLOSEBUTTONMASK, 0, 0, TBF_STRETCH };
	LOADABLE_THEME_BITMAP ltbAquaMinButton = { IDB_AQUA_MINBUTTON, IDB_AQUA_CLOSEBUTTONMASK, 0, 0, TBF_STRETCH };
	LOADABLE_THEME theme_aqua[] = {
		{ TCID_DEFAULT, TTID_FGCOLOR, TSID_NORMAL, THEME_DATA_COLOR(RGB(0,0,0)) },
		{ TCID_DEFAULT, TTID_FGCOLOR, TSID_DISABLED, THEME_DATA_COLOR(RGB(128,128,128)) },
		{ TCID_BUTTON, TTID_BITMAP, TSID_NORMAL, THEME_DATA_BITMAP(&ltbAquaButton) },
		{ TCID_BUTTON, TTID_BITMAP, TSID_FOCUSED, THEME_DATA_BITMAP(&ltbAquaButtonFocus) },
		{ TCID_BUTTON, TTID_BITMAP, TSID_SELECTED, THEME_DATA_BITMAP(&ltbAquaButtonFocus) },
		{ TCID_CHECKBOX, TTID_CHECKED, TSID_NORMAL, THEME_DATA_BITMAP(&ltbAquaCheck) },
		{ TCID_CHECKBOX, TTID_UNCHECKED, TSID_NORMAL, THEME_DATA_BITMAP(&ltbAquaUncheck) },
		{ TCID_PROGRESSBAR, TTID_BITMAP, TSID_NORMAL, THEME_DATA_BITMAP(&ltbAquaProgressBG) },
		{ TCID_PROGRESSBAR, TTID_FGBITMAP, TSID_NORMAL, THEME_DATA_BITMAP(&ltbAquaProgressFG) },
		{ TCID_COMBOBOX, TTID_DROPBUTTON, TSID_NORMAL, THEME_DATA_BITMAP(&ltbAquaDropButton) },
		{ TCID_COMBOBOX, TTID_BITMAP, TSID_NORMAL, THEME_DATA_BITMAP(&ltbAquaComboBoxBG) },
		{ TCID_DIALOG, TTID_BGBITMAP, TSID_NORMAL, THEME_DATA_BITMAP(&ltbAquaDialog) },
		{ TCID_DIALOG, TTID_TITLEFGCOLOR, TSID_NORMAL, THEME_DATA_COLOR(RGB(0,0,0)) },
		{ TCID_DIALOG, TTID_TITLEBITMAP, TSID_NORMAL, THEME_DATA_BITMAP(&ltbAquaTitleActive) },
		{ TCID_DIALOG, TTID_TITLEBITMAP, TSID_DISABLED, THEME_DATA_BITMAP(&ltbAquaTitleInactive) },
		{ TCID_DIALOG, TTID_CLOSEBUTTON, TSID_NORMAL, THEME_DATA_BITMAP(&ltbAquaCloseButton) },
		{ TCID_DIALOG, TTID_MAXBUTTON, TSID_NORMAL, THEME_DATA_BITMAP(&ltbAquaMaxButton) },
		{ TCID_DIALOG, TTID_RESTOREBUTTON, TSID_NORMAL, THEME_DATA_BITMAP(&ltbAquaMaxButton) },
		{ TCID_DIALOG, TTID_MINBUTTON, TSID_NORMAL, THEME_DATA_BITMAP(&ltbAquaMinButton) },
		{ TCID_END, 0, 0, NULL }
	};
#if 1
	LOADABLE_THEME_BITMAP ltbTowadaButton      = { IDB_TOWADA_BUTTON, 0, 6, 4, TBF_STRETCH };
	LOADABLE_THEME_BITMAP ltbTowadaButtonFocus = { IDB_TOWADA_BUTTONFOCUS, 0, 6, 4, TBF_STRETCH };
	LOADABLE_THEME_BITMAP ltbTowadaButtonSelect= { IDB_TOWADA_BUTTONSELECT, 0, 6, 4, TBF_STRETCH };
	LOADABLE_THEME_BITMAP ltbTowadaChecked     = { IDB_TOWADA_CHECKED, 0, 4, 4, TBF_STRETCH };
	LOADABLE_THEME_BITMAP ltbTowadaUnchecked   = { IDB_TOWADA_UNCHECKED, 0, 4, 4, TBF_STRETCH };
	LOADABLE_THEME_BITMAP ltbTowadaDropButton  = { IDB_TOWADA_DROPBUTTON, 0, 4, 4, TBF_STRETCH };
	LOADABLE_THEME_BITMAP ltbTowadaDialog      = { IDB_TOWADA_DIALOGBG, 0, 25, 25, TBF_STRETCH };
	LOADABLE_THEME_BITMAP ltbTowadaProgressBG  = { IDB_TOWADA_PROGRESSBG, 0, 6, 4, TBF_VSTRETCH };
	LOADABLE_THEME_BITMAP ltbTowadaProgressFG  = { IDB_TOWADA_PROGRESSFG, 0, 6, 4, TBF_VSTRETCH };
	LOADABLE_THEME theme_towadako[] = {
		{ TCID_DEFAULT, TTID_FGCOLOR, TSID_NORMAL, THEME_DATA_COLOR(RGB(255,255,200)) },
		{ TCID_DEFAULT, TTID_FGCOLOR, TSID_DISABLED, THEME_DATA_COLOR(RGB(192,192,192)) },
		{ TCID_BUTTON, TTID_BITMAP, TSID_NORMAL, THEME_DATA_BITMAP(&ltbTowadaButton) },
		{ TCID_BUTTON, TTID_BITMAP, TSID_FOCUSED, THEME_DATA_BITMAP(&ltbTowadaButtonFocus) },
		{ TCID_BUTTON, TTID_BITMAP, TSID_SELECTED, THEME_DATA_BITMAP(&ltbTowadaButtonSelect) },
//		{ TCID_BUTTON, TTID_BITMAP, TSID_DISABLED, THEME_DATA_BITMAP(&ltbTowadaButtonDisable) },
		{ TCID_CHECKBOX, TTID_CHECKED, TSID_NORMAL, THEME_DATA_BITMAP(&ltbTowadaChecked) },
		{ TCID_CHECKBOX, TTID_UNCHECKED, TSID_NORMAL, THEME_DATA_BITMAP(&ltbTowadaUnchecked) },
//		{ TCID_COMBOBOX, TTID_BITMAP, TSID_NORMAL, THEME_DATA_BITMAP(&ltbComboBoxBG) },
		{ TCID_COMBOBOX, TTID_DROPBUTTON, TSID_NORMAL, THEME_DATA_BITMAP(&ltbTowadaDropButton) },
		{ TCID_PROGRESSBAR, TTID_BITMAP, TSID_NORMAL, THEME_DATA_BITMAP(&ltbTowadaProgressBG) },
		{ TCID_PROGRESSBAR, TTID_FGBITMAP, TSID_NORMAL, THEME_DATA_BITMAP(&ltbTowadaProgressFG) },
		{ TCID_DIALOG, TTID_BGBITMAP, TSID_NORMAL, THEME_DATA_BITMAP(&ltbTowadaDialog) },
//		{ TCID_DIALOG, TTID_CLOSEBUTTON, TSID_NORMAL, THEME_DATA_BITMAP(&ltbCloseButton) },
//		{ TCID_DIALOG, TTID_TITLEBITMAP, TSID_NORMAL, THEME_DATA_BITMAP(&ltbTitleActive) },
//		{ TCID_DIALOG, TTID_TITLEBITMAP, TSID_DISABLED, THEME_DATA_BITMAP(&ltbTitleInactive) },
		{ TCID_END, 0, 0, NULL }
	};
#endif
#if 0
	/* pu-nyan */
	LOADABLE_THEME_BITMAP ltbPunyanButton      = { IDB_PUNYAN_BUTTON, 0, 10, 2, TBF_STRETCH };
//	LOADABLE_THEME_BITMAP ltbPunyanButtonFocus = { IDB_PUNYAN_BUTTONFOCUS, 0, 10, 2, TBF_STRETCH };
//	LOADABLE_THEME_BITMAP ltbPunyanCheck       = { IDB_PUNYAN_CHECKED, IDB_PUNYAN_CHECKEDMASK, 0, 0, TBF_TILE };
//	LOADABLE_THEME_BITMAP ltbPunyanUncheck     = { IDB_PUNYAN_UNCHECKED, IDB_PUNYAN_UNCHECKEDMASK, 0, 0, TBF_TILE };
	LOADABLE_THEME_BITMAP ltbPunyanProgressBG  = { IDB_PUNYAN_PROGRESSBG, 0, 0, 0, TBF_VSTRETCH };
	LOADABLE_THEME_BITMAP ltbPunyanProgressFG  = { IDB_PUNYAN_PROGRESSFG, 0, 0, 0, TBF_VSTRETCH };
	LOADABLE_THEME_BITMAP ltbPunyanDropButton  = { IDB_PUNYAN_DROPBUTTON, 0, 0, 0, TBF_STRETCH };
	LOADABLE_THEME_BITMAP ltbPunyanComboBoxBG  = { IDB_PUNYAN_COMBOBOXBG, 0, 6, 2, TBF_VSTRETCH };
	LOADABLE_THEME_BITMAP ltbPunyanTitleActive = { IDB_PUNYAN_TITLEACT, 0, 0, 0, TBF_VSTRETCH };
//	LOADABLE_THEME_BITMAP ltbPunyanTitleInactive  = { IDB_PUNYAN_TITLEINACT, 0, 0, 0, TBF_VSTRETCH };
	LOADABLE_THEME_BITMAP ltbPunyanDialog      = { IDB_PUNYAN_DIALOGBG, 0, 0, 0, TBF_TILE };
	LOADABLE_THEME_BITMAP ltbPunyanCloseButton = { IDB_PUNYAN_CLOSEBUTTON, IDB_PUNYAN_CLOSEBUTTONMASK, 0, 0, TBF_STRETCH };
//	LOADABLE_THEME_BITMAP ltbPunyanMaxButton = { IDB_PUNYAN_MAXBUTTON, IDB_PUNYAN_CLOSEBUTTONMASK, 0, 0, TBF_STRETCH };
//	LOADABLE_THEME_BITMAP ltbPunyanMinButton = { IDB_PUNYAN_MINBUTTON, IDB_PUNYAN_CLOSEBUTTONMASK, 0, 0, TBF_STRETCH };
	LOADABLE_THEME theme_punyan[] = {
		{ TCID_DEFAULT, TTID_FGCOLOR, TSID_NORMAL, THEME_DATA_COLOR(RGB(0,0,0)) },
		{ TCID_DEFAULT, TTID_FGCOLOR, TSID_DISABLED, THEME_DATA_COLOR(RGB(128,128,128)) },
		{ TCID_BUTTON, TTID_BITMAP, TSID_NORMAL, THEME_DATA_BITMAP(&ltbPunyanButton) },
//		{ TCID_BUTTON, TTID_BITMAP, TSID_FOCUSED, THEME_DATA_BITMAP(&ltbPunyanButtonFocus) },
//		{ TCID_BUTTON, TTID_BITMAP, TSID_SELECTED, THEME_DATA_BITMAP(&ltbPunyanButtonFocus) },
//		{ TCID_CHECKBOX, TTID_CHECKED, TSID_NORMAL, THEME_DATA_BITMAP(&ltbPunyanCheck) },
//		{ TCID_CHECKBOX, TTID_UNCHECKED, TSID_NORMAL, THEME_DATA_BITMAP(&ltbPunyanUncheck) },
		{ TCID_PROGRESSBAR, TTID_BITMAP, TSID_NORMAL, THEME_DATA_BITMAP(&ltbPunyanProgressBG) },
		{ TCID_PROGRESSBAR, TTID_FGBITMAP, TSID_NORMAL, THEME_DATA_BITMAP(&ltbPunyanProgressFG) },
		{ TCID_COMBOBOX, TTID_DROPBUTTON, TSID_NORMAL, THEME_DATA_BITMAP(&ltbPunyanDropButton) },
		{ TCID_COMBOBOX, TTID_BITMAP, TSID_NORMAL, THEME_DATA_BITMAP(&ltbPunyanComboBoxBG) },
		{ TCID_DIALOG, TTID_BGBITMAP, TSID_NORMAL, THEME_DATA_BITMAP(&ltbPunyanDialog) },
		{ TCID_DIALOG, TTID_TITLEFGCOLOR, TSID_NORMAL, THEME_DATA_COLOR(RGB(255,255,255)) },
		{ TCID_DIALOG, TTID_TITLEBITMAP, TSID_NORMAL, THEME_DATA_BITMAP(&ltbPunyanTitleActive) },
//		{ TCID_DIALOG, TTID_TITLEBITMAP, TSID_DISABLED, THEME_DATA_BITMAP(&ltbPunyanTitleInactive) },
		{ TCID_DIALOG, TTID_CLOSEBUTTON, TSID_NORMAL, THEME_DATA_BITMAP(&ltbPunyanCloseButton) },
//		{ TCID_DIALOG, TTID_MAXBUTTON, TSID_NORMAL, THEME_DATA_BITMAP(&ltbAquaMaxButton) },
//		{ TCID_DIALOG, TTID_RESTOREBUTTON, TSID_NORMAL, THEME_DATA_BITMAP(&ltbAquaMaxButton) },
//		{ TCID_DIALOG, TTID_MINBUTTON, TSID_NORMAL, THEME_DATA_BITMAP(&ltbAquaMinButton) },
		{ TCID_END, 0, 0, NULL }
	};
#endif
	CThemeSet *pThemeClass = ((CEnbanFukusyaApp *)AfxGetApp())->GetThemeClass();
	LOADABLE_THEME *theme;
	
	switch(m_nThemeNumber){
	case 1:
		theme = theme_aqua;
		break;
#if 1
	case 2:
		theme = theme_towadako;
		break;
#endif
#if 0
	case 2:
		theme = theme_punyan;
		break;
#endif
	default:
		theme = theme_normal;
	}

	pThemeClass->InitializeTheme();
	pThemeClass->LoadThemeFromStruct(theme);
	m_BtnOK.SetTheme(pThemeClass);
	m_BtnExit.SetTheme(pThemeClass);
	m_Progress1.SetTheme(pThemeClass);
	m_Progress2.SetTheme(pThemeClass);
	m_CmbReader.SetTheme(pThemeClass);
	m_CmbWriter.SetTheme(pThemeClass);
	m_StcReader.SetTheme(pThemeClass);
	m_StcWriter.SetTheme(pThemeClass);
	m_StcProgress1.SetTheme(pThemeClass);
	m_StcProgress2.SetTheme(pThemeClass);
	m_StcInfo.SetTheme(pThemeClass);
	this->SetTheme(pThemeClass);

	// Srbg}bv
	if(m_bmpLogo.m_hObject!=NULL){
		m_bmpLogo.DeleteObject();
		m_bmpLogoMask.DeleteObject();
	}
	m_bmpLogo.LoadBitmap(IDB_LOGO);
	m_bmpLogoMask.LoadBitmap(IDB_LOGOMASK);
}

void CEnbanFukusyaDlg::DrawLogo()
{
	CClientDC dc(this);
	RECT rcWnd, rcLogo;
	SIZE szLogo;
	THEME_BITMAP tb;
	CGraphicUtil GUtil;

	tb.bitmap = &m_bmpLogo;
	tb.mask = &m_bmpLogoMask;
	tb.border_height = 0;
	tb.border_height = 0;
	tb.flags = TBF_TILE;

	szLogo = GUtil.GetBitmapSize(&m_bmpLogo);

	GetClientRect(&rcWnd);
	GetDlgItem(IDC_STC_BORDER1)->GetWindowRect(&rcLogo);
	ScreenToClient(&rcLogo);

	rcLogo.bottom = rcLogo.top;
	rcLogo.left = (rcWnd.right-rcWnd.left - szLogo.cx)/2;
	rcLogo.right = rcLogo.left + szLogo.cx;
	rcLogo.top = rcLogo.bottom - szLogo.cy;
	GUtil.DrawThemeBitmap(this, &dc, &tb, &rcLogo);
}

LRESULT CEnbanFukusyaDlg::OnKickIdle(WPARAM wParam, LPARAM lParam)
{
	SCAN_t *drives;
	CString csMessage;
	CComboBox *cb;
	int nCount;
	int nDrive;

	if(m_bScanDone)
		return 0;

	CThemeSet *pThemeClass = ((CEnbanFukusyaApp *)AfxGetApp())->GetThemeClass();
	CThemeMessageDlg MsgDlg;
	MsgDlg.SetTheme(pThemeClass);

	for(nCount=0; nCount<2; nCount++){
		cb = nCount==0 ? &m_CmbReader : &m_CmbWriter;
		if(cb->GetCount()==0){
			drives = ScanDrive(nCount>0);
			if(drives==NULL){
				csMessage.LoadString(IDS_DRIVENOTFOUND);
				MsgDlg.Disp(csMessage, NULL, MB_ICONERROR);
				return 0;
			}
			for(nDrive=0; strlen((LPCTSTR)drives[nDrive].inqdata)!=0; nDrive++){
				cb->AddString((LPCTSTR)drives[nDrive].inqdata);
			}
			if(nCount==0){
				if(m_pReaderDrives!=NULL)
					free(m_pReaderDrives);
				m_pReaderDrives = drives;
			}
			else{
				if(m_pWriterDrives!=NULL)
					free(m_pWriterDrives);
				m_pWriterDrives = drives;
			}
			cb->SetCurSel(0);
		}
	}

	if(m_CmbWriter.GetCount() >= 2){
		m_CmbWriter.AddString("L^pfBXN}ς̑Su");
		m_nAllWritableDrives = m_CmbWriter.GetCount()-1;
	}

	m_CmbReader.AddString("ISOC[Wt@C");
	m_nReadDriveISO = m_CmbReader.GetCount()-1;
	m_CmbWriter.AddString("ISOC[Wt@C");
	m_nWriteDriveISO = m_CmbWriter.GetCount()-1;

	m_CmbReader.AddString("~ՃC[Wt@C");
	m_nReadDriveEmg = m_CmbReader.GetCount()-1;
	m_CmbWriter.AddString("~ՃC[Wt@C");
	m_nWriteDriveEmg = m_CmbWriter.GetCount()-1;

	m_CmbReader.AddString("lbg[N");
	m_nReadDriveNet = m_CmbReader.GetCount()-1;
	m_CmbWriter.AddString("lbg[N");
	m_nWriteDriveNet = m_CmbWriter.GetCount()-1;

	/*
	 * R}hCɂǎ摕uwmF
	 */
#define IS_DEVICE_ISO(s) \
		((strlen(s) > 4) && (stricmp(s+strlen(s)-4, ".iso")==0))
#define IS_DEVICE_EMG(s) \
		((strlen(s) > 4) && (stricmp(s+strlen(s)-4, ".emg")==0))
	CString csTmp;
	LPCTSTR lpszTmp;
	lpszTmp = GetArgReader();
	if(lpszTmp != NULL){
		if(IS_DEVICE_ISO(lpszTmp)){
			csTmp.Format("ISO(%s)", lpszTmp);
			m_CmbReader.DeleteString(m_nReadDriveISO);
			m_CmbReader.InsertString(m_nReadDriveISO, csTmp);
			m_CmbReader.SetCurSel(m_nReadDriveISO);
			m_CmbReader.EnableWindow(FALSE);
		}
		else if(IS_DEVICE_EMG(lpszTmp)){
			csTmp.Format("~ՃC[W(%s)", lpszTmp);
			m_CmbReader.DeleteString(m_nReadDriveEmg);
			m_CmbReader.InsertString(m_nReadDriveEmg, csTmp);
			m_CmbReader.SetCurSel(m_nReadDriveEmg);
			m_CmbReader.EnableWindow(FALSE);
		}
	}
	/*
	 * R}hCɂ鏑uwmF
	 */
	lpszTmp = GetArgWriter();
	if(lpszTmp != NULL){
		if(IS_DEVICE_ISO(lpszTmp)){
			csTmp.Format("ISO(%s)", lpszTmp);
			m_CmbWriter.DeleteString(m_nWriteDriveISO);
			m_CmbWriter.InsertString(m_nWriteDriveISO, csTmp);
			m_CmbWriter.SetCurSel(m_nWriteDriveISO);
			m_CmbWriter.EnableWindow(FALSE);
		}
		else if(IS_DEVICE_EMG(lpszTmp)){
			csTmp.Format("~ՃC[W(%s)", lpszTmp);
			m_CmbWriter.DeleteString(m_nWriteDriveEmg);
			m_CmbWriter.InsertString(m_nWriteDriveEmg, csTmp);
			m_CmbWriter.SetCurSel(m_nWriteDriveEmg);
			m_CmbWriter.EnableWindow(FALSE);
		}
	}

	m_bScanDone = TRUE;

	return 0;
}


BOOL CEnbanFukusyaDlg::DestroyWindow() 
{
	// TODO: ̈ʒuɌŗL̏ǉ邩A܂͊{NXĂяoĂ
	CloseAspi();
	if(m_pReaderDrives!=NULL){
		free(m_pReaderDrives);
		m_pReaderDrives=NULL;
	}
	if(m_pWriterDrives!=NULL){
		free(m_pWriterDrives);
		m_pWriterDrives=NULL;
	}

	return CDialog::DestroyWindow();
}

void CEnbanFukusyaDlg::OnCancel() 
{
	// TODO: ̈ʒuɓʂȌ㏈ǉĂB
	if(m_bRunning){
		UISetAbort();
		m_bRunning=FALSE;
		m_BtnOK.EnableWindow(FALSE);
		return;
	}
	SaveIniFile();
	CDialog::OnCancel();
}

void CEnbanFukusyaDlg::OnOK() 
{
	// TODO: ̈ʒuɂ̑̌ؗp̃R[hǉĂ
	
	//CDialog::OnOK();
	if(m_bRunning){
		UISetAbort();
		m_bRunning=FALSE;
		m_BtnOK.EnableWindow(FALSE);
		return;
	}

	CString csTemp;
	CThemeMessageDlg MsgDlg;
	CThemeSet *pThemeClass = ((CEnbanFukusyaApp *)AfxGetApp())->GetThemeClass();
	int ret;
	DRIVEID reader, *writerp=NULL;
	int sel_reader, sel_writer;
	//BOOL bOneDrive = FALSE;
	int nWriter;
	int nIndex;

	MsgDlg.SetTheme(pThemeClass);

	// IhCu擾
	sel_reader = m_CmbReader.GetCurSel();
	if(sel_reader==CB_ERR)
		return;
	sel_writer = m_CmbWriter.GetCurSel();
	if(sel_writer==CB_ERR)
		return;

	writerp = (DRIVEID *)malloc(sizeof(DRIVEID));
	if(writerp==NULL){
		csTemp.LoadString(IDS_DRIVEINITERROR);
		MsgDlg.Disp(csTemp, NULL, MB_ICONSTOP);
		return;
	}

	if(sel_writer == m_nAllWritableDrives){
		nWriter=0;
		for(nIndex=0; strlen((LPCTSTR)m_pWriterDrives[nIndex].inqdata)!=0; nIndex++){
			if(m_pReaderDrives[sel_reader].hid == m_pWriterDrives[nIndex].hid &&
				m_pReaderDrives[sel_reader].tid == m_pWriterDrives[nIndex].tid){
				/* ǎ摕u͖ */
				continue;
			}
			writerp = (DRIVEID *)realloc(writerp, (nWriter+1)*sizeof(DRIVEID));
			if(writerp==NULL){
				csTemp.LoadString(IDS_DRIVEINITERROR);
				MsgDlg.Disp(csTemp, NULL, MB_ICONSTOP);
				return;
			}
			writerp[nWriter].hid = m_pWriterDrives[nIndex].hid;
			writerp[nWriter].tid = m_pWriterDrives[nIndex].tid;
			nWriter++;
		}
	}
	else if(sel_writer == m_nWriteDriveISO){
		writerp->hid = HID_VIRTUAL;
		writerp->tid = CMDDRVTYPE_ISO;
		nWriter=1;
	}
	else if(sel_writer == m_nWriteDriveEmg){
		writerp->hid = HID_VIRTUAL;
		writerp->tid = CMDDRVTYPE_IMAGE;
		nWriter=1;
	}
	else if(sel_writer == m_nWriteDriveNet){
		writerp->hid = HID_VIRTUAL;
		writerp->tid = CMDDRVTYPE_NET;
		nWriter=1;
	}
	else{
		writerp->hid = m_pWriterDrives[sel_writer].hid;
		writerp->tid = m_pWriterDrives[sel_writer].tid;
		nWriter=1;
	}
	
	if(sel_reader == m_nReadDriveISO){
		reader.hid = HID_VIRTUAL;
		reader.tid = CMDDRVTYPE_ISO;
	}
	else if(sel_reader == m_nReadDriveEmg){
		reader.hid = HID_VIRTUAL;
		reader.tid = CMDDRVTYPE_IMAGE;
	}
	else if(sel_reader == m_nReadDriveNet){
		reader.hid = HID_VIRTUAL;
		reader.tid = CMDDRVTYPE_NET;
	}
	else{
		reader.hid = m_pReaderDrives[sel_reader].hid;
		reader.tid = m_pReaderDrives[sel_reader].tid;
	}

	UIClearAbort();
	UISetHWND(this->m_hWnd);
	m_bRunning=TRUE;
	m_BtnExit.EnableWindow(FALSE);
	m_CmbReader.EnableWindow(FALSE);
	m_CmbWriter.EnableWindow(FALSE);
	csTemp.LoadString(IDS_ABORT);
	m_BtnOK.SetWindowText(csTemp);
	m_BtnOK.SetFocus();

	ret = CopyDisc(&reader, writerp, nWriter);

	csTemp.LoadString(IDS_COPYSTART);
	m_BtnOK.SetWindowText(csTemp);
	m_BtnOK.EnableWindow(TRUE);
	m_BtnExit.EnableWindow(TRUE);
	m_CmbReader.EnableWindow(TRUE);
	m_CmbWriter.EnableWindow(TRUE);
	m_bRunning=FALSE;

	/*  */
	free(writerp);

	SendMessage(WM_USER_UICTL, UICTL_METER2INIT, NULL);
	SendMessage(WM_USER_UICTL, UICTL_METER1INIT, NULL);
}

LRESULT CEnbanFukusyaDlg::OnUICtl(WPARAM wParam, LPARAM lParam)
{
	LPCTSTR lpMessage;

	switch(wParam){
	case UICTL_METER1INIT:
		m_Progress1.SetPos(0);
		m_Progress1.SetRange(0, 1000);
		if(lParam==NULL){
			lpMessage = "";
			ShowMeter1(FALSE);
		}
		else{
			lpMessage = (LPCTSTR)lParam;
			ShowMeter1(TRUE);
		}
		m_StcProgress1.SetWindowText(lpMessage);
		break;
	case UICTL_METER2INIT:
		m_Progress2.SetPos(0);
		m_Progress2.SetRange(0, 1000);
		if(lParam==NULL){
			lpMessage = "";
			ShowMeter2(FALSE);
		}
		else{
			lpMessage = (LPCTSTR)lParam;
			ShowMeter2(TRUE);
		}
		m_StcProgress2.SetWindowText(lpMessage);
		break;
	case UICTL_METER1SETPOS:
		m_Progress1.SetPosAnim((int)lParam);
		TRACE("meter1=%ld\n", lParam);
		break;
	case UICTL_METER2SETPOS:
		m_Progress2.SetPosAnim((int)lParam);
		TRACE("meter2=%ld\n", lParam);
		break;
	case UICTL_DISPINFO:
		if(lParam==NULL){
			lpMessage = "";
			ShowInfo(FALSE);
		}
		else{
			lpMessage = (LPCTSTR)lParam;
			ShowInfo(TRUE);
		}
		m_StcInfo.SetWindowText(lpMessage);
		break;
	case UICTL_SETTING:
		return Setting((OPTIONS *)lParam);
	case UICTL_GETARGREADER:
		return (LONG)GetArgReader();
	case UICTL_GETARGWRITER:
		return (LONG)GetArgWriter();
	}

	return 0;
}

void CEnbanFukusyaDlg::ResizeWindow(int nStep)
{
	RECT rcChild, rcWnd;

	if(nStep==m_nWindowSizeStep)
		return;

	GetWindowRect(&rcWnd);
	ScreenToClient(&rcWnd);
	switch(nStep){
	case 0:
		m_Progress1.GetWindowRect(&rcChild);
		ScreenToClient(&rcChild);
		rcWnd.bottom = rcChild.top;
		break;
	case 1:
		m_Progress2.GetWindowRect(&rcChild);
		ScreenToClient(&rcChild);
		rcWnd.bottom = rcChild.top;
		break;
	case 2:
		m_Progress2.GetWindowRect(&rcChild);
		ScreenToClient(&rcChild);
		rcWnd.bottom = rcChild.bottom + 20;
		break;
	case 3:
		m_StcInfo.GetWindowRect(&rcChild);
		ScreenToClient(&rcChild);
		rcWnd.bottom = rcChild.bottom + 10;
		break;
	default:
		return;
	}
	SetWindowPos(NULL, 0, 0, rcWnd.right-rcWnd.left, rcWnd.bottom-rcWnd.top,
					SWP_NOZORDER|SWP_NOMOVE);
	m_nWindowSizeStep = nStep;
}


void CEnbanFukusyaDlg::ShowMeter1(BOOL bShow)
{

	if(bShow){
		if(m_nWindowSizeStep<1)
			ResizeWindow(1);
	}
	else{
		ResizeWindow(0);
	}
}

void CEnbanFukusyaDlg::ShowMeter2(BOOL bShow)
{
	if(bShow){
		if(m_nWindowSizeStep<2)
			ResizeWindow(2);
	}
	else{
		ResizeWindow(1);
	}
}

void CEnbanFukusyaDlg::ShowInfo(BOOL bShow)
{
	if(bShow){
		ResizeWindow(3);
	}
	else{
		ResizeWindow(2);
	}
}

int CEnbanFukusyaDlg::Setting(OPTIONS *pOption)
{
	CSettingDlg dlg;
	int ret;

	dlg.SetTheme(GetTheme());
	dlg.SetData(pOption);
	ret = dlg.DoModal();
	return ret==IDOK ? RET_OK : RET_ABORT;
}

void CEnbanFukusyaDlg::LoadIniFile()
{
	char buf[80];

	GetRegString("theme", buf, sizeof(buf));
	if(!strcmp(buf, "towadako")){
		m_nThemeNumber = 2;
	}
	else if(!strcmp(buf, "aqua")){
		m_nThemeNumber = 1;
	}
	else{
		m_nThemeNumber = 0;
	}
}

void CEnbanFukusyaDlg::SaveIniFile()
{
	char *type;

	switch(m_nThemeNumber){
	case 1:
		type = "aqua";
		break;
	case 2:
		type = "towadako";
		break;
	default:
		type = "normal";
	}
	WriteRegString("theme", type);
}

void CEnbanFukusyaDlg::SetArgReader(LPCTSTR lpszReader)
{
	m_csArgReader = lpszReader;
}

LPCTSTR CEnbanFukusyaDlg::GetArgReader()
{
	return m_csArgReader;
}

void CEnbanFukusyaDlg::SetArgWriter(LPCTSTR lpszWriter)
{
	m_csArgWriter = lpszWriter;
}


LPCTSTR CEnbanFukusyaDlg::GetArgWriter()
{
	return m_csArgWriter;
}

